<h2>The Detox Framework Reference Manual</h2>

<div id="documentation">
<ol id="index">
	<li><a href="#intro">Introduction</a>
		<ol>
			<li><a href="#overview">Overview</a></li>
			<li><a href="#goals">Design goals</a></li>
			<li><a href="#requirements">Requirements</a>
				<ol>
					<li><a href="#system-requirements">System requirements</a></li>
					<li><a href="#technical-requirements">Technical requirements</a></li>
				</ol>
			</li>
			<li><a href="#usage-scenarios">Usage scenarios</a></li>
			<li><a href="#urimapping">How URIs are mapped</a></li>
			<li><a href="#autodetox">AutoDetox: Detox code generation tool</a></li>
		</ol>
	</li>
	<li><a href="#first-app">The First Detox Application</a>
		<ol>
			<li><a href="#first-db">The database</a></li>
			<li><a href="#first-model">Creating the model</a></li>
			<li><a href="#first-controller">Creating the controller</a></li>
			<li><a href="#first-views">Creating the views</a></li>
			<li><a href="#first-extending">Extending the application</a></li>
		</ol>
	</li>
	<li><a href="#architeture">Architeture</a>
		<ol>
			<li><a href="#controller">The <span>Controller</span> class</a></li>
			<li><a href="#appcontroller">The <span>ApplicationController</span> class</a></li>
			<li><a href="#customcontrollers">User-defined controllers</a></li>
			<li><a href="#requesthandling">How requests are handled</a></li>
		</ol>
	</li>
	<li><a href="#ormapping">Object-Relational Mapping</a>
		<ol>
			<li><a href="#ormapper">The Detox OR Mapper</a></li>
			<li><a href="#sql">Writing your own SQL</a></li>
		</ol>
	</li>
	<li><a href="#about">About this documentation</a></li>
	<li><a href="#license">License</a></li>
</ol>

<!-- INTRODUCTION -->
<h3 id="intro">Introduction</h2>
<!-- overview -->
<h4 id="overview">Overview</h4>
<p>Detox is a <strong>MVC framework</strong> built in and for the <strong>PHP programming language</strong>. Detox
aims to ease and speed-up the development process of PHP web applications by providing a clean MVC structure and the basic classes for controllers, models and database connections.</p>
<p>Customize the framework to match your application needs should be an easy task, through some PHP skills are required. Having some basic knowledge of how the MVC design pattern works is also a good thing.
<p>Please notice that Detox is still a <em>work in progress</em>, in its very initial state, and still have lots of loose ends. For the next versions we are focusing on:
<ul>
	<li>Make a decent i18n support for code generation/OR Mapping</li>
	<li>Make a better object relational mapper</li>
	<li><strong>Make a decent implementation of AutoDetox</strong></li>
	<li>Simplify the way objects are created</li>
	<li>Give more flexibility to the way data is sent to the views</li>
</ul>
<p>And more. Any questions or suggestions should be sent to rspenha@users.sourceforge.net</p>
<p><strong>Also notice that this documentation itself is a work in progress, and corrections and contributions are welcome.</strong></p>
<!--/overview -->
<!-- design goals -->
<h4 id="goals">Design goals</h4>
<p>Detox was built with speed and performance as a main design goal. We tried to eliminate configuration files and statements by using heavy object orientation and introspection, ensuring that your application will have <strong>less files to open and perform includes</strong>, wich are a costly operation. The price is that you will have to get your hands in PHP code sometimes. If you dontt <strong>just love PHP programming</strong> or are not familiar with the concepts mentioned above, dontt worry; all code you will need to write is really simple, and most of it can be generated by AutoDetox.</p>
<p>We think that writing simple code in a programming language you already know, like PHP, is better than learning new commands and executing then in a command-line interface (to later end up probably having to put your hands on code anyway). The Detox Framework was built with PHP programmers in mind (or at least people that know a little PHP), and we are sure that your application will be better (and faster) if you automatize just some parts of its development (like the creation of models, controllers and basic views) and write its specific code like login validations and so (if you need to) by yourself, anyway you want or are used to do it.</p>
<p>Simple deployment was also a design goal. Detox do not use any operating system or web server vendor-specific function, and do not depend on any external API or library - so deploying a Detox application is simple as deploying any PHP application (usually something like a copy & paste process).</p>
<p>Two of the main focus of Detox are simplicity and ease of use. Simplicity is being achieved; the core framework is about 20kb and defining a new action to be mapped from a URI (and then presenting a view or doing anything else) is easy as a function definition, sometimes a one-line function. Ease of use is provided by clear standards and good documentation; as Detox is still under a very initial state, it is lacking from both in many cases. We are trying to correct this for the next versions.</p>
<!--/design goals -->
<!-- requirements -->
<h4 id="requirements">Requirements</h4>
<h5 id="system-requirements">System requirements</h5>
<p>Detox is compatible with PHP4/5 and currently was tested in Apache 2.x, but should be compatible with IIS and other Apache versions, as well as other servers, as it do not use any vendor-specific function. This also
applies for operating systems. Testing Detox in multiple environement configurations still an issue; maybe you can <a href="contact">help us</a> by telling your experience with Detox in your environement (specially if it was a bad experience).</p>
<p>Detox version 0.2.3 currently supports MySQL databases (tested with MySQL 5.0.45). Other databases can be used, but you will need to write the database wrapper for yourself.</p>
<h5 id="technical-requirements">Technical requirements</h5>
<p>To use Detox properly you should have some basic skills in PHP, as you will need to write some pieces of code. Detox has some code generation capabilities (with AutoDetox), but you will always need to get your hands in the code to fully build your application. We tried to make the framework as simple as possible so anyone with a little knowledge of PHP and web development in general could work with it with no trouble.</p>
<h4 id="usage-scenarios">Usage scenarios</h4>
<p>Detox can be used to build a variety of web applications, but focus mainly in database-driven web applications. It can be used to build content-management systems, portals, e-commerce systems and so on, from small to large applications. It can easily work with other tools (like mod_rewrite) to extend the framework capabilities and the end-user experience.</p>
<p>Detox can also be used to build non-database-driven websites (like this one), taking advantages of all of its templating system capabilities. <a href="http://phpdetox.sourceforge.net">The Detox Website</a> was built with a simplified version of Detox and can be <a href="http://phpdetox.sourceforge.net/detoxwebsite.zip">downloaded</a> for inspiration on what you can do with the framework.</p>
<!--/requirements -->
<!-- uri mapping -->
<h4 id="urimapping">How URIs are mapped</h4>
<p>All requests are handled by the ApplicationController object in the index.php file. All URIs must follow this pattern: <strong>?c=controller&a=action&d=data</strong></p>
<p>You may easily alter the way your URIs look using mod_rewrite or other URI rewriting engine. More information on this should be provided with new versions of this documentation.</p>
<!--/uri mapping -->
<!-- autodetox -->
<h4 id="autodetox">AutoDetox</h4>
<p>AutoDetox is the standard code generation tool for Detox. It reads a given database and then write the models,
views and controllers for it (the code for the sample application below was generated by AutoDetox).
AutoDetox aims to speed-up the development process of web applications by taking care of the repetitive work.</p>
<p>To try AutoDetox, run <strong>yourhost/autodetox/</strong> in your browser and read the instructions.</p>
<p><strong>Remeber that AutoDetox is in a experimental version, and we do not ensure that it will work properly. Also remember to delete the /autodetox folder before deploying your application, as it may be viewed by anyone and turn into a potential danger for your application.</strong></p>
<!--/autodetox -->
<!--/INTRODUCTION -->

<!--/FIRST APP -->
<h3 id="first-app">The First Detox Application</h3>
<h4 id="first-db">The database</h4>
<p>In Detox version 0.2.3 all tables need a primary key called 'id' (case sensitive!), int, auto_increment. We did it this way because it was <em>easy</em>, but it is an ugly limitation and is likely to be removed in future releases
(so your database may be designed anyway you want).</p>
<p>The sample application will use a database with a single table. Create the following database:</p>
<pre>
create database detoxtest;
use detoxtest;
CREATE TABLE users (
	id int auto_increment,
	name varchar(80),
	email varchar(80),
	PRIMARY KEY (id)
);
</pre>
<p>Then edit the <strong>core/db/connector.php</strong> file to match your database connection. The file looks like this:</p>
<pre>
require_once("MySQLConnector.php");

class Connector extends MySQLConnector {

	var $host ='localhost';
	var $user ='root';
	var $password = '';
	var $database = 'detoxtest';

}
</pre>

<h4 id="first-model">Creating the model</h4>
<p>The models are placed inside <strong>core/models/</strong>. The files must be named $classname.php
(<strong>user.php</strong> for table users, <strong>article.php</strong> for table articles, and so on). Remenber
to keep it on singular when writing your controllers!.</p>
<p>This is how user.php should look like:</p>
<pre>
&lt;?php

require_once("recordable.php");
require_once("factory.php");

class UserFactory extends Factory {

	var $model = "user";

	function build($array) {
		return new User($array);
	}

}

class User extends Recordable {

	var $id;
	var $name;
	var $email;

	function User($array) {
		$this->id = $array["id"];
		$this->name = $array["name"];
		$this->email = $array["email"];
	}

}

?&gt;
</pre>
<p>It consists of a factory for User instances a definition of the User class. The constructor takes an array
as argument and start the object attributes with the values in the array corresponding to the attribute name.</p>


<h4 id="first-controller">Creating the controller</h4>
<p>The controller is responsible for handling the requests and presenting views to the user. The <span>ApplicationController</span> object at index.php receives the request, then call the corresponding controller, that then handle the request. All controllers should inherit from the <span>Controller</span> class.</p>
<p>This is how the controller for user should look like:</p>
<pre>
require_once("models/user.php");

class UserController extends Controller {

	function UserController() {
		$this->factory = new UserFactory();
		$this->model = "user";
		$this->defaultAction = "list";
	}

}

$appcontrol = new UserController();
</pre>
<p>defaultAction represents the default action that will be performed when the action is not specified in the URI.
If defaultAction is not defined, then the base Controller class default action will be used (pointing to 'index').</p>

<h4 id="first-views">Creating the views</h4>
<p>One advantage of the Model-View-Controller design pattern is that your application pages can be created as "dumb templates"; pages that just present data and have absolutely no idea about business logic. This can ease the job web designers and ther people responsible for interface implementation as they will not need to face much PHP code while creating the pages.</p>
<p>By default, all data is sent to the views in a variable called $data. This variable can be a object instance from a model or a collection of objects, depending on the situation. This will be clearer as we write the basic views to operate on our <span>user</span> table.</p>
<h5>The <strong>list</strong> view</h5>
<p>The code for the list view presented by the user controller when the "list" action is requested (and, in this case, also when no action is requested) is as follows.</p>
<p>To see it in action, go to <strong>?c=user&a=list</strong> in your application. As you defined in your <span>UserController</span>, this view will also be presented when no action is requested (in a URI like <strong>?c=user)</strong></p>
<pre>
&lt;table cellpadding="0" cellspacing="0"&gt;
	&lt;tr&gt;
		&lt;th>id&lt;/th&gt;
		&lt;th&gt;name&lt;/th&gt;
		&lt;th&gt;email&lt;/th&gt;
		&lt;th&gt;username&lt;/th&gt;
		&lt;th&gt;psw&lt;/th&gt;&lt;th&gt;Delete&lt;/th&gt;
	&lt;/tr&gt;
	&lt;?php 
		if ($data) {
			foreach ($data as $obj) {
				echo "&lt;tr&gt;";
				echo "&lt;td&gt;".$obj-&gt;id."&lt;/td&gt;";
				echo "&lt;td&gt;".$obj-&gt;name."&lt;/td&gt;";
				echo "&lt;td&gt;".$obj-&gt;email."&lt;/td&gt;";
				echo "&lt;td&gt;".$obj-&gt;username."&lt;/td&gt;";
				echo "&lt;td&gt;".$obj-&gt;psw."&lt;/td&gt;";
				echo "&lt;td&gt;&lt;a href=?c=user&a=delete&d=".$obj->id."&gt;delete&lt;/a&gt;&lt;/td&gt;&lt;/tr&gt;";
				echo "&lt;td&gt;&lt;a href=?c=user&a=edit&d=".$obj-&gt;id."&gt;edit&lt;/a&gt;&lt;/td&gt;";
			}
		}else {
			echo "&lt;tr&gt;&lt;td&gt;No results.&lt;/td&gt;&lt;/tr&gt;";
		}
	?&gt;
</table>
</pre>
<h5>The <strong>create</strong> view</h5>
<p>This is the form for creating users. The code is as follows. Notice that the form points to the <span>user</span> controller, calling the <span>new</span> action (the same URI of the page that presents the user creation form). All forms in Detox use the <strong>post</strong> method. This view is just standard XHTML.</p>
<p>After creating this file, go to <strong>?c=user&a=new</strong> to see it in action.</p>
<pre>
&lt;form id="user" action="?c=user&a=new" method="post"&gt;
&lt;fieldset&gt;
&lt;legend&gt;New User&lt;/legend&gt;
&lt;ul&gt;
	&lt;li&gt;
		&lt;li&gt;
			&lt;label for="name"&gt;name&lt;/label&gt;
			&lt;input type="text" id="name" name="name" /&gt;
		&lt;/li&gt;
		&lt;li&gt;
		&lt;li&gt;
			&lt;label for="email"&gt;email&lt;/label&gt;
			&lt;input type="text" id="email" name="email" /&gt;
		&lt;/li&gt;
		&lt;li&gt;
		&lt;li&gt;
			&lt;label for="username"&gt;username&lt;/label&gt;
			&lt;input type="text" id="username" name="username" /&gt;
		&lt;/li&gt;
		&lt;li&gt;
		&lt;li&gt;
			&lt;label for="psw"&gt;psw&lt;/label&gt;
			&lt;input type="text" id="psw" name="psw" /&gt;
		&lt;/li&gt;
		&lt;li class="buttons"&gt;
			&lt;input type="submit" value="Submit" /&gt;
		&lt;/li&gt;
	&lt;/ul&gt;
&lt;/fieldset&gt;
&lt;/form&gt;
</pre>

<h5>The <strong>edit</strong> view</h5>
<p>This is the form for editing users. The code is as follows. Notice that the form points to the <span>user</span> controller, calling the <span>edit</span> action, with the data refered being the id of the current database record (the same URI of the page that presents this user edit form). This view is just standard XHTML with little PHP code to present the current values for this user record.</p>
<pre>
&lt;form id="user" action="?c=user&a=edit&d=&lt;?php echo $data-&gt;id ?&gt;" method="post"&gt;
&lt;fieldset&gt;
&lt;legend&gt;Edit User&lt;/legend&gt;
&lt;ul&gt;
	&lt;li&gt;
		&lt;li&gt;
			&lt;label for="name"&gt;name&lt;/label&gt;
			&lt;input type="text" id="name" name="name" value="&lt;?php echo $data-&gt;name ?&gt;" /&gt;
		&lt;/li&gt;
		&lt;li&gt;
		&lt;li&gt;
			&lt;label for="email"&gt;email&lt;/label&gt;
			&lt;input type="text" id="email" name="email" value="&lt;?php echo $data-&gt;email ?&gt;" /&gt;
		&lt;/li&gt;
		&lt;li&gt;
		&lt;li&gt;
			&lt;label for="username"&gt;username&lt;/label&gt;
			&lt;input type="text" id="username" name="username" value="&lt;?php echo $data-&gt;username ?&gt;" /&gt;
		&lt;/li&gt;
		&lt;li&gt;
		&lt;li&gt;
			&lt;label for="psw"&gt;psw&lt;/label&gt;
			&lt;input type="text" id="psw" name="psw" value="&lt;?php echo $data-&gt;psw ?&gt;" /&gt;
		&lt;/li&gt;
		&lt;li class="buttons"&gt;
			&lt;input type="submit" value="Submit" /&gt;
		&lt;/li&gt;
	&lt;/ul&gt;
&lt;/fieldset&gt;
&lt;/form&gt;
</pre>


<h3 id="first-extending">Extending the application</h3>
<p>You can define new actions for your controllers. Every action is a function in the controller class.
If you want to create an action to map 'something' (in a URI like ?c=user&a=something), add this method
to the controller class:</p>
<pre>
require_once("models/user.php");

class UserController extends Controller {

	function UserController() {
		$this->factory = new UserFactory();
		$this->model = "user";
		$this->defaultAction = "list";
	}

	<strong class="new">
	function handle_something() {
		$this->present('some string');
	}
	</strong>

}

$appcontrol = new UserController();
</pre>

<p>If now you access <strong>?c=user&a=something</strong>, a screen with your website template with
'some string' written in the content part should appear in your browser. This is a simple way to demonstrate
how to create new actions; they can call other functions, start/delete session variables, present views, redirect
the user and do anything else your normal PHP code does.</p>
<p><strong>Notice the syntax!</strong> The method to handle an action must start with 'handle_', followed by
a lowercase version of the action name.</p>
<p>Ex: To handle a <strong>login</strong> action in the user controller,
you should create a method named <strong>handle_login</strong>, and then access it by <strong>?c=user&a=login</strong>.
<!--/FIRST APP -->

<!-- ARCHITETURE -->
<h3 id="architeture">Detox Architeture</h3>
<p>Detox was build using the Model-View-Controller design pattern. This way it decouples presentation from business logic and data models, being the industry standard architeture for web applications. More on the MVC design pattern should be provided with new versions of this documentation.</p>
<!-- controller -->
<h4 id="controller">The <span>Controller</span> class</h4>
<p>The <span>Controller</span> class is located in <strong>/core/controller.php</strong> and is the base class for all controllers in the application. It contains the methods to present views (files or other contents) and handle requests. All controller classes you create should extend this one; if you dont want to do that, notice that the class you create will need at least to have a method named <span>handle()</span>, as it is called from the ApplicationController when handling the requests.</p>
<!--/controller -->
<!-- application -->
<h4 id="appcontroller">The <span>ApplicationController</span> class</h4>
<p>This is the class responsible for parsing the requests and calling all other controllers. It is defined in /core/application.php and by default only one object from this class exists; the one in the <strong>index.php</strong> file in your application root folder.</p>
<p>Any code you write inside the ApplicationController constructor will be performed every time a request is performed to your application. This is a simple way to perform tasks like login check and more.</p>
<!--/application -->
<!-- other controllers -->
<h4 id="customcontrollers">User-defined controllers</h4>
<p>Any user-defined controller must be placed in a file on the <strong>/core</strong> folder. There must be only one controller per file, and an object of the controller should be created after its class definition in a variable named <span>$appcontrol</span>. The <span>ApplicationController</span> object in the index.php file will
use by default the controller recorded in <span>$appcontrol</span> to handle the request after dynamically including the controller file.</p>
<!--/other controllers -->
<!-- request handling -->
<h4 id="requesthandling">How requests are handled</h4>
<p>All requests for your application must be directed to the index.php file in the root folder of your application. The request is then parsed by the ApplicationController object, that dynamically include the corresponding controller file for your request (if your request was <strong>?c=user</strong>, the ApplicationController will call <strong>/core/user.php</strong>).</p>
<p>The included controller file have a definition of the controller class (UserController, for example) and an instantiation of this class as $appcontrol. The controller instance in $appcontrol is then used by the ApplicationController to handle the action in the request. If the action does not correspond to a method in the controller called in the URI, then a 404 error will happen. A 404 will also happen if the user request a unexisting controller. The 404 page can be customized in <strong>/views/404.html</strong>.</p>
<!--/request handling -->
<!--/ARCHITETURE-->

<!-- OR MAPPING -->
<h3 id="ormapping">Object-Relational Mapping</h3>
<p>Detox offer some object-relational mapping capabilities for its models with the <span>Recordable</span> class. Any class that you wish to use with Detox object-relational mapper should extend the Recordable class in the <strong>/core/models/recordable.php</strong> file. These classes will have by default methods to handle <strong>create, list, update and delete</strong> actions and will use instrospection to check your model object state and record it on the database. These classes also will need views to support these actions (what you are going to show in this views is up to you).</p>
<h4 id="ormapper">The Detox OR Mapper</h4>
<p>While OR mapping can save much development time, the OR mapping functions in Detox are still under heavy development, and may become an independent project. We looked for some PHP OR Mapper, but all of them seemed to be too bloated. OR Mapping is one of the major concerns for the next versions, and for now you can try to use it, but if it do not work properly, just consider about writing the SQL for your models by yourself. It is not a hard task at all; just take a look below.</p>
<h4 id="sql">Writing your own SQL</h4>
<p>To write your own SQL for your models, just <strong>do not extend</strong> the <span>Recordable</span> class while creating your models classes, and write the methods to handle the operations in the database (list, create, edit, delete) by yourself.</p>
<p>In this case you will need to include the <strong>core/db/connector.php</strong> file in your model file, and create an instance of the <span>Connector</span> before executing any database operation.</p>
<p>To execute the sql, call the <span>executeQuery</span> method of the Connector object you created. Something like this:</p>
<pre>
$conn = new Connector();
$results = $conn->executeQuery('select * from users');
</pre>
<!--/OR MAPPING -->

<!-- ABOUT -->
<h3 id="about">About this documentation</h3>
<p>This reference manual is a work in progress, and is part of an effort to document and give directions to the development of future versions of the Detox framework. Notice that this was not written by a native english speaker and <a href="mailto:rspenha@users.sourceforge.net">corrections and contributions</a> are welcome; translations are also welcome, but notice that this documentation may suffer big changes for the next versions (as the framework itself).</p>
<!--/ABOUT -->

<!-- LICENSE -->
<h3 id="license">License</h3>
<p>The Detox Framework is free software and can be used, distributed and changed under the terms of the BSD License.</p>
<pre>
Copyright (c) 2008, Rafael da Silva Rocha <rspenha@users.sourceforge.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holder nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
</pre>
<!--/LICENSE -->

</div>